@@ -1,125 +1,73 @@
 // VolEsti (volume computation and sampling library)
 
-// Copyright (c) 20012-2018 Vissarion Fisikopoulos
-// Copyright (c) 2018 Apostolos Chalkis
+// Copyright (c) 2012-2020 Vissarion Fisikopoulos
+// Copyright (c) 2018-2020 Apostolos Chalkis
 
 //Contributed and/or modified by Apostolos Chalkis, as part of Google Summer of Code 2018 program.
+//Contributed and/or modified by Repouskos Panagiotis, as part of Google Summer of Code 2019 program.
 
 // Licensed under GNU LGPL.3, see LICENCE file
 
 #ifndef BALLINTERSECTCONVEX_H
 #define BALLINTERSECTCONVEX_H
 
-// ball type
-template <class Point>
-struct Ball{
-public:
-    typedef Point BallPoint;
-    typedef typename Point::FT NT;
-    typedef typename std::vector<NT>::iterator viterator;
 
-
-    Ball(Point cc, NT RR) : c(cc),	 R(RR) {}
-
-    Point center(){
-        return c;
-    }
-
-    NT squared_radius(){
-        return R;
-    }
-
-    NT radius(){
-        return std::sqrt(R);
-    }
-
-    int is_in(Point p){
-        if (p.squared_length() <= R)
-            return -1;
-        else return 0;
-    }
-
-    std::pair<NT,NT> line_intersect(Point r,
-                                          Point v){
-
-        viterator rit=r.iter_begin();
-        viterator vit=v.iter_begin();
-        viterator cit=c.iter_begin();
-        //Point rc = r;// - _c;
-        viterator rcit=r.iter_begin();
-        NT vrc(0);
-        NT v2(0);
-        NT rc2(0);
-        for( ; cit < c.iter_end() ; ++rcit, ++cit, ++rit, ++vit){
-            vrc += *vit * (*rcit);
-            v2 += *vit * (*vit);
-            rc2 += *rcit * (*rcit);
-        }
-
-        NT disc_sqrt = std::sqrt(std::pow(vrc,2) - v2 * (rc2 - R));
-        NT lamda1((NT(-1)*vrc + disc_sqrt)/v2);
-        NT lamda2((NT(-1)*vrc - disc_sqrt)/v2);
-        return std::pair<NT,NT> (lamda1,lamda2);
-    }
-
-    std::pair<NT,NT> line_intersect_coord(Point r,
-                                          int rand_coord){
-
-        //Point rc = r;// - _c;
-        viterator rcit=r.iter_begin();
-        NT vrc = *(rcit + rand_coord);
-
-        //NT v2 = NT(1);
-        NT rc2(R);
-        for( ; rcit < r.iter_end() ; ++rcit){
-            rc2 -= *rcit * (*rcit);
-        }
-
-        //NT disc_sqrt = std::sqrt(std::pow(vrc,2) - v2 * (rc2 - _R));
-        NT disc_sqrt = std::sqrt(std::pow(vrc,2) + rc2);// + _R);
-        NT lamda1((NT(-1)*vrc + disc_sqrt));
-        NT lamda2((NT(-1)*vrc - disc_sqrt));
-
-        return std::pair<NT,NT> (lamda1,lamda2);
-
-    }
-
-private:
-    Point  c; //center
-    NT     R; //SQUARED radius !!!
-};
-
-
-template <class Polytope, class CBall>
+template <typename Polytope, typename CBall>
 class BallIntersectPolytope {
 private:
     Polytope    P;
     CBall B;
 public:
+    typedef typename Polytope::MT MT;
+    typedef typename Polytope::VT VT;
     typedef typename CBall::NT NT;
-    typedef typename CBall::BallPoint Point;
+    typedef typename CBall::PointType PointType;
 
-    BallIntersectPolytope(Polytope &PP, CBall &BB) : P(PP), B(BB) {};
-    
-    Polytope first() { return P; }
-    CBall second() { return B; }
+    BallIntersectPolytope() {}
 
-    int is_in(Point p){
-        if(B.is_in(p)==-1)
+    BallIntersectPolytope(Polytope const& PP, CBall &BB) : P(PP), B(BB) {};
+
+    Polytope first() const { return P; }
+    CBall second() const { return B; }
+
+    std::pair<PointType,NT> InnerBall() const
+    {
+        return P.InnerBall();
+    }
+
+    MT get_mat() const {
+        return P.get_mat();
+    }
+
+    MT get_T() const {
+        return P.get_mat();
+    }
+
+    MT get_vec() const {
+        return P.get_vec();
+    }
+
+    int is_in(PointType const& p) const
+    {
+        if (B.is_in(p)==-1)
             return P.is_in(p);
         return 0;
     }
 
-    int num_of_hyperplanes(){
+    int num_of_hyperplanes() const {
         return P.num_of_hyperplanes();
     }
 
-    unsigned int dimension(){
+    unsigned int dimension() const {
         return P.dimension();
     }
 
-    std::pair<NT,NT> line_intersect(Point r,
-                                          Point v) {
+    NT radius() const {
+        return B.radius();
+    }
+
+    std::pair<NT,NT> line_intersect(PointType const& r, PointType const& v) const
+    {
 
         std::pair <NT, NT> polypair = P.line_intersect(r, v);
         std::pair <NT, NT> ballpair = B.line_intersect(r, v);
@@ -127,10 +75,66 @@ public:
                                  std::max(polypair.second, ballpair.second));
     }
 
+    std::pair<NT,NT> line_intersect(PointType const& r,
+                                    PointType const& v,
+                                    VT &Ar,
+                                    VT &Av) const
+    {
+        std::pair <NT, NT> polypair = P.line_intersect(r, v, Ar, Av);
+        std::pair <NT, NT> ballpair = B.line_intersect(r, v);
+        return std::pair<NT, NT>(std::min(polypair.first, ballpair.first),
+                                 std::max(polypair.second, ballpair.second));
+    }
+
+    std::pair<NT,NT> line_intersect(PointType const& r,
+                                    PointType const& v,
+                                    VT &Ar,
+                                    VT &Av,
+                                    NT &lambda_prev) const
+    {
+        std::pair <NT, NT> polypair = P.line_intersect(r, v, Ar, Av, lambda_prev);
+        std::pair <NT, NT> ballpair = B.line_intersect(r, v);
+        return std::pair<NT, NT>(std::min(polypair.first, ballpair.first),
+                                 std::max(polypair.second, ballpair.second));
+    }
+
+    std::pair<NT,int> line_positive_intersect(PointType const& r,
+                                              PointType const& v,
+                                              VT &Ar,
+                                              VT &Av) const
+    {
+
+        std::pair <NT, int> polypair = P.line_positive_intersect(r, v, Ar, Av);
+        std::pair <NT, int> ball_lambda = B.line_positive_intersect(r, v);
+        int facet = P.num_of_hyperplanes();
+
+        if (polypair.first < ball_lambda.first ) facet = polypair.second;
+
+        return std::pair<NT, int>(std::min(polypair.first, ball_lambda.first), facet);
+    }
+
+
+    std::pair<NT,int> line_positive_intersect(PointType const& r,
+                                              PointType const& v,
+                                              VT &Ar,
+                                              VT &Av,
+                                              NT &lambda_prev) const
+    {
+
+        std::pair <NT, int> polypair = P.line_positive_intersect(r, v, Ar, Av, lambda_prev);
+        std::pair <NT, int> ball_lambda = B.line_positive_intersect(r, v);
+        int facet = P.num_of_hyperplanes();
+
+        if (polypair.first < ball_lambda.first ) facet = polypair.second;
+
+        return std::pair<NT, int>(std::min(polypair.first, ball_lambda.first), facet);
+    }
+
     //First coordinate ray shooting intersecting convex body
-    std::pair<NT,NT> line_intersect_coord(Point &r,
-                                          unsigned int rand_coord,
-                                          std::vector<NT> &lamdas) {
+    std::pair<NT,NT> line_intersect_coord(PointType const& r,
+                                          unsigned int const& rand_coord,
+                                          VT &lamdas) const
+    {
 
         std::pair <NT, NT> polypair = P.line_intersect_coord(r, rand_coord, lamdas);
         std::pair <NT, NT> ballpair = B.line_intersect_coord(r, rand_coord);
@@ -139,25 +143,39 @@ public:
     }
 
     //Not the first coordinate ray shooting intersecting convex body
-    std::pair<NT,NT> line_intersect_coord(Point &r,
-                                          Point &r_prev,
-                                          unsigned int rand_coord,
-                                          unsigned int rand_coord_prev,
-                                          std::vector<NT> &lamdas) {
-
-        std::pair <NT, NT> polypair = P.line_intersect_coord(r, r_prev, rand_coord, rand_coord_prev, lamdas);
+    std::pair<NT,NT> line_intersect_coord(PointType const& r,
+                                          PointType const& r_prev,
+                                          unsigned int const& rand_coord,
+                                          unsigned int const& rand_coord_prev,
+                                          VT &lamdas) const
+    {
+        std::pair <NT, NT> polypair = P.line_intersect_coord(r, r_prev, rand_coord,
+                                                             rand_coord_prev, lamdas);
         std::pair <NT, NT> ballpair = B.line_intersect_coord(r, rand_coord);
         return std::pair<NT, NT>(std::min(polypair.first, ballpair.first),
                                  std::max(polypair.second, ballpair.second));
     }
 
-    std::pair<NT,NT> query_dual(Point &p, unsigned int rand_coord) {
+    std::pair<NT,NT> query_dual(PointType const& p,
+                                unsigned int const& rand_coord)
+    {
         std::pair <NT, NT> polypair = P.query_dual(p, rand_coord);
         std::pair <NT, NT> ballpair = B.line_intersect_coord(p, rand_coord);
         return std::pair<NT, NT>(std::min(polypair.first, ballpair.first),
                                  std::max(polypair.second, ballpair.second));
     }
 
+    void compute_reflection (PointType& v, PointType const& p, int &facet) const
+    {
+
+        if (facet == P.num_of_hyperplanes()) {
+            B.compute_reflection(v, p, facet);
+        } else {
+            P.compute_reflection(v, p, facet);
+        }
+
+    }
+
 };
 
 
@@ -170,17 +188,17 @@ private:
     typedef typename T2::K 	K;
 public:
     PolytopeIntersectEllipsoid(T1 &Pin, T2 &Ein) : P(Pin), E(Ein) {};
-    
+
     T1 first() { return P; }
     T2 second() { return E; }
-    
+
     int is_in(Point p){
         //std::cout << "calling is in"<<std::endl;
         if(P.is_in(p)==-1)
             return E.is_in(p);
         return 0;
     }
-    
+
     int num_of_hyperplanes(){
         return P.num_of_hyperplanes();
     }
@@ -188,7 +206,7 @@ public:
     int dimension(){
         return P.dimension();
     }
-    
+
     std::pair<Point,Point> line_intersect(Point r,
                                           Point v){
 
@@ -216,7 +234,7 @@ public:
         }
         return returnpair;
     }
-    
+
     std::pair<K,K> line_intersect_coord(Point &r,
                                           Point &r_prev,
                                           int rand_coord,
@@ -230,7 +248,7 @@ public:
         return std::pair<K,K> (std::min(polypair.first,ellpair.first),
                                  std::max(polypair.second,ellpair.second));
     }
-    
+
 };
 
 
@@ -242,17 +260,17 @@ private:
     typedef typename T2::K 	K;
 public:
     BallPolyIntersectEll(T1 &BPin, T2 &Ein) : BP(BPin), E(Ein) {};
-    
+
     T1 first() { return BP; }
     T2 second() { return E; }
-    
+
     int is_in(Point p){
         //std::cout << "calling is in"<<std::endl;
         if(BP.is_in(p)==-1)
             return E.is_in(p);
         return 0;
     }
-    
+
     int num_of_hyperplanes(){
         return BP.num_of_hyperplanes();
     }
@@ -260,7 +278,7 @@ public:
     int dimension(){
         return BP.dimension();
     }
-    
+
     std::pair<Point,Point> line_intersect(Point r,
                                           Point v){
 
@@ -312,9 +330,9 @@ public:
         return std::pair<K,K> (std::min(Bpolypair.first,ellpair.first),
                                  std::max(Bpolypair.second,ellpair.second));
     }
-    
-    
-    
+
+
+
 };*/
 
 #endif
